# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/contrib/rpc/python/kernel_tests/test_example.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import types_pb2 as tensorflow_dot_core_dot_framework_dot_types__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow/contrib/rpc/python/kernel_tests/test_example.proto',
  package='tensorflow.contrib.rpc',
  syntax='proto2',
  serialized_pb=_b('\n=tensorflow/contrib/rpc/python/kernel_tests/test_example.proto\x12\x16tensorflow.contrib.rpc\x1a%tensorflow/core/framework/types.proto\"\x9d\x01\n\x08TestCase\x12\x41\n\tprimitive\x18\x01 \x03(\x0b\x32..tensorflow.contrib.rpc.RepeatedPrimitiveValue\x12\r\n\x05shape\x18\x02 \x03(\x05\x12\r\n\x05sizes\x18\x03 \x03(\x05\x12\x30\n\x05\x66ield\x18\x04 \x03(\x0b\x32!.tensorflow.contrib.rpc.FieldSpec\"\x80\x01\n\tFieldSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12#\n\x05\x64type\x18\x02 \x01(\x0e\x32\x14.tensorflow.DataType\x12@\n\x08\x65xpected\x18\x03 \x01(\x0b\x32..tensorflow.contrib.rpc.RepeatedPrimitiveValue\"\xe0\x02\n\tTestValue\x12?\n\x0fprimitive_value\x18\x01 \x01(\x0b\x32&.tensorflow.contrib.rpc.PrimitiveValue\x12\x35\n\nenum_value\x18\x02 \x01(\x0b\x32!.tensorflow.contrib.rpc.EnumValue\x12;\n\rmessage_value\x18\x03 \x01(\x0b\x32$.tensorflow.contrib.rpc.MessageValue\x12L\n\x16repeated_message_value\x18\x04 \x01(\x0b\x32,.tensorflow.contrib.rpc.RepeatedMessageValue\x12P\n\x18repeated_primitive_value\x18\x06 \x01(\x0b\x32..tensorflow.contrib.rpc.RepeatedPrimitiveValue\"\xda\x02\n\x0ePrimitiveValue\x12\x14\n\x0c\x64ouble_value\x18\x01 \x01(\x01\x12\x13\n\x0b\x66loat_value\x18\x02 \x01(\x02\x12\x13\n\x0bint64_value\x18\x03 \x01(\x03\x12\x14\n\x0cuint64_value\x18\x04 \x01(\x04\x12\x13\n\x0bint32_value\x18\x05 \x01(\x05\x12\x15\n\rfixed64_value\x18\x06 \x01(\x06\x12\x15\n\rfixed32_value\x18\x07 \x01(\x07\x12\x12\n\nbool_value\x18\x08 \x01(\x08\x12\x14\n\x0cstring_value\x18\t \x01(\t\x12\x13\n\x0b\x62ytes_value\x18\x0c \x01(\x0c\x12\x14\n\x0cuint32_value\x18\r \x01(\r\x12\x16\n\x0esfixed32_value\x18\x0f \x01(\x0f\x12\x16\n\x0esfixed64_value\x18\x10 \x01(\x10\x12\x14\n\x0csint32_value\x18\x11 \x01(\x11\x12\x14\n\x0csint64_value\x18\x12 \x01(\x12\"\xa1\x03\n\x16RepeatedPrimitiveValue\x12\x14\n\x0c\x64ouble_value\x18\x01 \x03(\x01\x12\x13\n\x0b\x66loat_value\x18\x02 \x03(\x02\x12\x13\n\x0bint64_value\x18\x03 \x03(\x03\x12\x14\n\x0cuint64_value\x18\x04 \x03(\x04\x12\x13\n\x0bint32_value\x18\x05 \x03(\x05\x12\x15\n\rfixed64_value\x18\x06 \x03(\x06\x12\x15\n\rfixed32_value\x18\x07 \x03(\x07\x12\x12\n\nbool_value\x18\x08 \x03(\x08\x12\x14\n\x0cstring_value\x18\t \x03(\t\x12\x13\n\x0b\x62ytes_value\x18\x0c \x03(\x0c\x12\x14\n\x0cuint32_value\x18\r \x03(\r\x12\x16\n\x0esfixed32_value\x18\x0f \x03(\x0f\x12\x16\n\x0esfixed64_value\x18\x10 \x03(\x10\x12\x14\n\x0csint32_value\x18\x11 \x03(\x11\x12\x14\n\x0csint64_value\x18\x12 \x03(\x12\x12=\n\rmessage_value\x18\x13 \x03(\x0b\x32&.tensorflow.contrib.rpc.PrimitiveValue\"\xd3\x03\n\x14PackedPrimitiveValue\x12\x18\n\x0c\x64ouble_value\x18\x01 \x03(\x01\x42\x02\x10\x01\x12\x17\n\x0b\x66loat_value\x18\x02 \x03(\x02\x42\x02\x10\x01\x12\x17\n\x0bint64_value\x18\x03 \x03(\x03\x42\x02\x10\x01\x12\x18\n\x0cuint64_value\x18\x04 \x03(\x04\x42\x02\x10\x01\x12\x17\n\x0bint32_value\x18\x05 \x03(\x05\x42\x02\x10\x01\x12\x19\n\rfixed64_value\x18\x06 \x03(\x06\x42\x02\x10\x01\x12\x19\n\rfixed32_value\x18\x07 \x03(\x07\x42\x02\x10\x01\x12\x16\n\nbool_value\x18\x08 \x03(\x08\x42\x02\x10\x01\x12\x14\n\x0cstring_value\x18\t \x03(\t\x12\x13\n\x0b\x62ytes_value\x18\x0c \x03(\x0c\x12\x18\n\x0cuint32_value\x18\r \x03(\rB\x02\x10\x01\x12\x1a\n\x0esfixed32_value\x18\x0f \x03(\x0f\x42\x02\x10\x01\x12\x1a\n\x0esfixed64_value\x18\x10 \x03(\x10\x42\x02\x10\x01\x12\x18\n\x0csint32_value\x18\x11 \x03(\x11\x42\x02\x10\x01\x12\x18\n\x0csint64_value\x18\x12 \x03(\x12\x42\x02\x10\x01\x12=\n\rmessage_value\x18\x13 \x03(\x0b\x32&.tensorflow.contrib.rpc.PrimitiveValue\"\xe5\x01\n\tEnumValue\x12;\n\nenum_value\x18\x0e \x01(\x0e\x32\'.tensorflow.contrib.rpc.EnumValue.Color\x12\x44\n\x13repeated_enum_value\x18\x0f \x03(\x0e\x32\'.tensorflow.contrib.rpc.EnumValue.Color\"U\n\x05\x43olor\x12\x07\n\x03RED\x10\x00\x12\n\n\x06ORANGE\x10\x01\x12\n\n\x06YELLOW\x10\x02\x12\t\n\x05GREEN\x10\x03\x12\x08\n\x04\x42LUE\x10\x04\x12\n\n\x06INDIGO\x10\x05\x12\n\n\x06VIOLET\x10\x06\">\n\x11InnerMessageValue\x12\x13\n\x0b\x66loat_value\x18\x02 \x01(\x02\x12\x14\n\x0c\x62ytes_values\x18\x08 \x03(\x0c\"\x82\x01\n\x12MiddleMessageValue\x12\x14\n\x0cint32_values\x18\x05 \x03(\x05\x12@\n\rmessage_value\x18\x0b \x01(\x0b\x32).tensorflow.contrib.rpc.InnerMessageValue\x12\x14\n\x0cuint32_value\x18\r \x01(\r\"g\n\x0cMessageValue\x12\x14\n\x0c\x64ouble_value\x18\x01 \x01(\x01\x12\x41\n\rmessage_value\x18\x0b \x01(\x0b\x32*.tensorflow.contrib.rpc.MiddleMessageValue\"\xb0\x01\n\x14RepeatedMessageValue\x12W\n\x0emessage_values\x18\x0b \x03(\x0b\x32?.tensorflow.contrib.rpc.RepeatedMessageValue.NestedMessageValue\x1a?\n\x12NestedMessageValue\x12\x13\n\x0b\x66loat_value\x18\x02 \x01(\x02\x12\x14\n\x0c\x62ytes_values\x18\x08 \x03(\x0c\"9\n\x0b\x45xtraFields\x12\x15\n\x0cstring_value\x18\xf0\r \x01(\t\x12\x13\n\nbool_value\x18\xf1\r \x01(\x08\x32\xf4\x03\n\x0fTestCaseService\x12[\n\x13IncrementTestShapes\x12 .tensorflow.contrib.rpc.TestCase\x1a .tensorflow.contrib.rpc.TestCase\"\x00\x12T\n\x0cSleepForever\x12 .tensorflow.contrib.rpc.TestCase\x1a .tensorflow.contrib.rpc.TestCase\"\x00\x12]\n\x15SometimesSleepForever\x12 .tensorflow.contrib.rpc.TestCase\x1a .tensorflow.contrib.rpc.TestCase\"\x00\x12\x65\n\x1d\x41lwaysFailWithInvalidArgument\x12 .tensorflow.contrib.rpc.TestCase\x1a .tensorflow.contrib.rpc.TestCase\"\x00\x12h\n SometimesFailWithInvalidArgument\x12 .tensorflow.contrib.rpc.TestCase\x1a .tensorflow.contrib.rpc.TestCase\"\x00')
  ,
  dependencies=[tensorflow_dot_core_dot_framework_dot_types__pb2.DESCRIPTOR,])



_ENUMVALUE_COLOR = _descriptor.EnumDescriptor(
  name='Color',
  full_name='tensorflow.contrib.rpc.EnumValue.Color',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RED', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ORANGE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='YELLOW', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GREEN', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BLUE', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INDIGO', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VIOLET', index=6, number=6,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2158,
  serialized_end=2243,
)
_sym_db.RegisterEnumDescriptor(_ENUMVALUE_COLOR)


_TESTCASE = _descriptor.Descriptor(
  name='TestCase',
  full_name='tensorflow.contrib.rpc.TestCase',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='primitive', full_name='tensorflow.contrib.rpc.TestCase.primitive', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shape', full_name='tensorflow.contrib.rpc.TestCase.shape', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sizes', full_name='tensorflow.contrib.rpc.TestCase.sizes', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='field', full_name='tensorflow.contrib.rpc.TestCase.field', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=129,
  serialized_end=286,
)


_FIELDSPEC = _descriptor.Descriptor(
  name='FieldSpec',
  full_name='tensorflow.contrib.rpc.FieldSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.contrib.rpc.FieldSpec.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dtype', full_name='tensorflow.contrib.rpc.FieldSpec.dtype', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expected', full_name='tensorflow.contrib.rpc.FieldSpec.expected', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=289,
  serialized_end=417,
)


_TESTVALUE = _descriptor.Descriptor(
  name='TestValue',
  full_name='tensorflow.contrib.rpc.TestValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='primitive_value', full_name='tensorflow.contrib.rpc.TestValue.primitive_value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='enum_value', full_name='tensorflow.contrib.rpc.TestValue.enum_value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_value', full_name='tensorflow.contrib.rpc.TestValue.message_value', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeated_message_value', full_name='tensorflow.contrib.rpc.TestValue.repeated_message_value', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeated_primitive_value', full_name='tensorflow.contrib.rpc.TestValue.repeated_primitive_value', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=420,
  serialized_end=772,
)


_PRIMITIVEVALUE = _descriptor.Descriptor(
  name='PrimitiveValue',
  full_name='tensorflow.contrib.rpc.PrimitiveValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='double_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.double_value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.float_value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int64_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.int64_value', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint64_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.uint64_value', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int32_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.int32_value', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fixed64_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.fixed64_value', index=5,
      number=6, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fixed32_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.fixed32_value', index=6,
      number=7, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bool_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.bool_value', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='string_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.string_value', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.bytes_value', index=9,
      number=12, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint32_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.uint32_value', index=10,
      number=13, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sfixed32_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.sfixed32_value', index=11,
      number=15, type=15, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sfixed64_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.sfixed64_value', index=12,
      number=16, type=16, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sint32_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.sint32_value', index=13,
      number=17, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sint64_value', full_name='tensorflow.contrib.rpc.PrimitiveValue.sint64_value', index=14,
      number=18, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=775,
  serialized_end=1121,
)


_REPEATEDPRIMITIVEVALUE = _descriptor.Descriptor(
  name='RepeatedPrimitiveValue',
  full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='double_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.double_value', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.float_value', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int64_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.int64_value', index=2,
      number=3, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint64_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.uint64_value', index=3,
      number=4, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int32_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.int32_value', index=4,
      number=5, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fixed64_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.fixed64_value', index=5,
      number=6, type=6, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fixed32_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.fixed32_value', index=6,
      number=7, type=7, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bool_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.bool_value', index=7,
      number=8, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='string_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.string_value', index=8,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.bytes_value', index=9,
      number=12, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint32_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.uint32_value', index=10,
      number=13, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sfixed32_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.sfixed32_value', index=11,
      number=15, type=15, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sfixed64_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.sfixed64_value', index=12,
      number=16, type=16, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sint32_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.sint32_value', index=13,
      number=17, type=17, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sint64_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.sint64_value', index=14,
      number=18, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_value', full_name='tensorflow.contrib.rpc.RepeatedPrimitiveValue.message_value', index=15,
      number=19, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1124,
  serialized_end=1541,
)


_PACKEDPRIMITIVEVALUE = _descriptor.Descriptor(
  name='PackedPrimitiveValue',
  full_name='tensorflow.contrib.rpc.PackedPrimitiveValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='double_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.double_value', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.float_value', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int64_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.int64_value', index=2,
      number=3, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint64_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.uint64_value', index=3,
      number=4, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int32_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.int32_value', index=4,
      number=5, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fixed64_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.fixed64_value', index=5,
      number=6, type=6, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fixed32_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.fixed32_value', index=6,
      number=7, type=7, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bool_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.bool_value', index=7,
      number=8, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='string_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.string_value', index=8,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.bytes_value', index=9,
      number=12, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint32_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.uint32_value', index=10,
      number=13, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sfixed32_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.sfixed32_value', index=11,
      number=15, type=15, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sfixed64_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.sfixed64_value', index=12,
      number=16, type=16, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sint32_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.sint32_value', index=13,
      number=17, type=17, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sint64_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.sint64_value', index=14,
      number=18, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_value', full_name='tensorflow.contrib.rpc.PackedPrimitiveValue.message_value', index=15,
      number=19, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1544,
  serialized_end=2011,
)


_ENUMVALUE = _descriptor.Descriptor(
  name='EnumValue',
  full_name='tensorflow.contrib.rpc.EnumValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enum_value', full_name='tensorflow.contrib.rpc.EnumValue.enum_value', index=0,
      number=14, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeated_enum_value', full_name='tensorflow.contrib.rpc.EnumValue.repeated_enum_value', index=1,
      number=15, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ENUMVALUE_COLOR,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2014,
  serialized_end=2243,
)


_INNERMESSAGEVALUE = _descriptor.Descriptor(
  name='InnerMessageValue',
  full_name='tensorflow.contrib.rpc.InnerMessageValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='float_value', full_name='tensorflow.contrib.rpc.InnerMessageValue.float_value', index=0,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_values', full_name='tensorflow.contrib.rpc.InnerMessageValue.bytes_values', index=1,
      number=8, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2245,
  serialized_end=2307,
)


_MIDDLEMESSAGEVALUE = _descriptor.Descriptor(
  name='MiddleMessageValue',
  full_name='tensorflow.contrib.rpc.MiddleMessageValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='int32_values', full_name='tensorflow.contrib.rpc.MiddleMessageValue.int32_values', index=0,
      number=5, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_value', full_name='tensorflow.contrib.rpc.MiddleMessageValue.message_value', index=1,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uint32_value', full_name='tensorflow.contrib.rpc.MiddleMessageValue.uint32_value', index=2,
      number=13, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2310,
  serialized_end=2440,
)


_MESSAGEVALUE = _descriptor.Descriptor(
  name='MessageValue',
  full_name='tensorflow.contrib.rpc.MessageValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='double_value', full_name='tensorflow.contrib.rpc.MessageValue.double_value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_value', full_name='tensorflow.contrib.rpc.MessageValue.message_value', index=1,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2442,
  serialized_end=2545,
)


_REPEATEDMESSAGEVALUE_NESTEDMESSAGEVALUE = _descriptor.Descriptor(
  name='NestedMessageValue',
  full_name='tensorflow.contrib.rpc.RepeatedMessageValue.NestedMessageValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='float_value', full_name='tensorflow.contrib.rpc.RepeatedMessageValue.NestedMessageValue.float_value', index=0,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_values', full_name='tensorflow.contrib.rpc.RepeatedMessageValue.NestedMessageValue.bytes_values', index=1,
      number=8, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2661,
  serialized_end=2724,
)

_REPEATEDMESSAGEVALUE = _descriptor.Descriptor(
  name='RepeatedMessageValue',
  full_name='tensorflow.contrib.rpc.RepeatedMessageValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message_values', full_name='tensorflow.contrib.rpc.RepeatedMessageValue.message_values', index=0,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_REPEATEDMESSAGEVALUE_NESTEDMESSAGEVALUE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2548,
  serialized_end=2724,
)


_EXTRAFIELDS = _descriptor.Descriptor(
  name='ExtraFields',
  full_name='tensorflow.contrib.rpc.ExtraFields',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_value', full_name='tensorflow.contrib.rpc.ExtraFields.string_value', index=0,
      number=1776, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bool_value', full_name='tensorflow.contrib.rpc.ExtraFields.bool_value', index=1,
      number=1777, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2726,
  serialized_end=2783,
)

_TESTCASE.fields_by_name['primitive'].message_type = _REPEATEDPRIMITIVEVALUE
_TESTCASE.fields_by_name['field'].message_type = _FIELDSPEC
_FIELDSPEC.fields_by_name['dtype'].enum_type = tensorflow_dot_core_dot_framework_dot_types__pb2._DATATYPE
_FIELDSPEC.fields_by_name['expected'].message_type = _REPEATEDPRIMITIVEVALUE
_TESTVALUE.fields_by_name['primitive_value'].message_type = _PRIMITIVEVALUE
_TESTVALUE.fields_by_name['enum_value'].message_type = _ENUMVALUE
_TESTVALUE.fields_by_name['message_value'].message_type = _MESSAGEVALUE
_TESTVALUE.fields_by_name['repeated_message_value'].message_type = _REPEATEDMESSAGEVALUE
_TESTVALUE.fields_by_name['repeated_primitive_value'].message_type = _REPEATEDPRIMITIVEVALUE
_REPEATEDPRIMITIVEVALUE.fields_by_name['message_value'].message_type = _PRIMITIVEVALUE
_PACKEDPRIMITIVEVALUE.fields_by_name['message_value'].message_type = _PRIMITIVEVALUE
_ENUMVALUE.fields_by_name['enum_value'].enum_type = _ENUMVALUE_COLOR
_ENUMVALUE.fields_by_name['repeated_enum_value'].enum_type = _ENUMVALUE_COLOR
_ENUMVALUE_COLOR.containing_type = _ENUMVALUE
_MIDDLEMESSAGEVALUE.fields_by_name['message_value'].message_type = _INNERMESSAGEVALUE
_MESSAGEVALUE.fields_by_name['message_value'].message_type = _MIDDLEMESSAGEVALUE
_REPEATEDMESSAGEVALUE_NESTEDMESSAGEVALUE.containing_type = _REPEATEDMESSAGEVALUE
_REPEATEDMESSAGEVALUE.fields_by_name['message_values'].message_type = _REPEATEDMESSAGEVALUE_NESTEDMESSAGEVALUE
DESCRIPTOR.message_types_by_name['TestCase'] = _TESTCASE
DESCRIPTOR.message_types_by_name['FieldSpec'] = _FIELDSPEC
DESCRIPTOR.message_types_by_name['TestValue'] = _TESTVALUE
DESCRIPTOR.message_types_by_name['PrimitiveValue'] = _PRIMITIVEVALUE
DESCRIPTOR.message_types_by_name['RepeatedPrimitiveValue'] = _REPEATEDPRIMITIVEVALUE
DESCRIPTOR.message_types_by_name['PackedPrimitiveValue'] = _PACKEDPRIMITIVEVALUE
DESCRIPTOR.message_types_by_name['EnumValue'] = _ENUMVALUE
DESCRIPTOR.message_types_by_name['InnerMessageValue'] = _INNERMESSAGEVALUE
DESCRIPTOR.message_types_by_name['MiddleMessageValue'] = _MIDDLEMESSAGEVALUE
DESCRIPTOR.message_types_by_name['MessageValue'] = _MESSAGEVALUE
DESCRIPTOR.message_types_by_name['RepeatedMessageValue'] = _REPEATEDMESSAGEVALUE
DESCRIPTOR.message_types_by_name['ExtraFields'] = _EXTRAFIELDS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TestCase = _reflection.GeneratedProtocolMessageType('TestCase', (_message.Message,), dict(
  DESCRIPTOR = _TESTCASE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.TestCase)
  ))
_sym_db.RegisterMessage(TestCase)

FieldSpec = _reflection.GeneratedProtocolMessageType('FieldSpec', (_message.Message,), dict(
  DESCRIPTOR = _FIELDSPEC,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.FieldSpec)
  ))
_sym_db.RegisterMessage(FieldSpec)

TestValue = _reflection.GeneratedProtocolMessageType('TestValue', (_message.Message,), dict(
  DESCRIPTOR = _TESTVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.TestValue)
  ))
_sym_db.RegisterMessage(TestValue)

PrimitiveValue = _reflection.GeneratedProtocolMessageType('PrimitiveValue', (_message.Message,), dict(
  DESCRIPTOR = _PRIMITIVEVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.PrimitiveValue)
  ))
_sym_db.RegisterMessage(PrimitiveValue)

RepeatedPrimitiveValue = _reflection.GeneratedProtocolMessageType('RepeatedPrimitiveValue', (_message.Message,), dict(
  DESCRIPTOR = _REPEATEDPRIMITIVEVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.RepeatedPrimitiveValue)
  ))
_sym_db.RegisterMessage(RepeatedPrimitiveValue)

PackedPrimitiveValue = _reflection.GeneratedProtocolMessageType('PackedPrimitiveValue', (_message.Message,), dict(
  DESCRIPTOR = _PACKEDPRIMITIVEVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.PackedPrimitiveValue)
  ))
_sym_db.RegisterMessage(PackedPrimitiveValue)

EnumValue = _reflection.GeneratedProtocolMessageType('EnumValue', (_message.Message,), dict(
  DESCRIPTOR = _ENUMVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.EnumValue)
  ))
_sym_db.RegisterMessage(EnumValue)

InnerMessageValue = _reflection.GeneratedProtocolMessageType('InnerMessageValue', (_message.Message,), dict(
  DESCRIPTOR = _INNERMESSAGEVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.InnerMessageValue)
  ))
_sym_db.RegisterMessage(InnerMessageValue)

MiddleMessageValue = _reflection.GeneratedProtocolMessageType('MiddleMessageValue', (_message.Message,), dict(
  DESCRIPTOR = _MIDDLEMESSAGEVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.MiddleMessageValue)
  ))
_sym_db.RegisterMessage(MiddleMessageValue)

MessageValue = _reflection.GeneratedProtocolMessageType('MessageValue', (_message.Message,), dict(
  DESCRIPTOR = _MESSAGEVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.MessageValue)
  ))
_sym_db.RegisterMessage(MessageValue)

RepeatedMessageValue = _reflection.GeneratedProtocolMessageType('RepeatedMessageValue', (_message.Message,), dict(

  NestedMessageValue = _reflection.GeneratedProtocolMessageType('NestedMessageValue', (_message.Message,), dict(
    DESCRIPTOR = _REPEATEDMESSAGEVALUE_NESTEDMESSAGEVALUE,
    __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.RepeatedMessageValue.NestedMessageValue)
    ))
  ,
  DESCRIPTOR = _REPEATEDMESSAGEVALUE,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.RepeatedMessageValue)
  ))
_sym_db.RegisterMessage(RepeatedMessageValue)
_sym_db.RegisterMessage(RepeatedMessageValue.NestedMessageValue)

ExtraFields = _reflection.GeneratedProtocolMessageType('ExtraFields', (_message.Message,), dict(
  DESCRIPTOR = _EXTRAFIELDS,
  __module__ = 'tensorflow.contrib.rpc.python.kernel_tests.test_example_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.contrib.rpc.ExtraFields)
  ))
_sym_db.RegisterMessage(ExtraFields)


_PACKEDPRIMITIVEVALUE.fields_by_name['double_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['double_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['float_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['float_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['int64_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['int64_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['uint64_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['uint64_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['int32_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['int32_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['fixed64_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['fixed64_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['fixed32_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['fixed32_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['bool_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['bool_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['uint32_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['uint32_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['sfixed32_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['sfixed32_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['sfixed64_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['sfixed64_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['sint32_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['sint32_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_PACKEDPRIMITIVEVALUE.fields_by_name['sint64_value'].has_options = True
_PACKEDPRIMITIVEVALUE.fields_by_name['sint64_value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))

_TESTCASESERVICE = _descriptor.ServiceDescriptor(
  name='TestCaseService',
  full_name='tensorflow.contrib.rpc.TestCaseService',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=2786,
  serialized_end=3286,
  methods=[
  _descriptor.MethodDescriptor(
    name='IncrementTestShapes',
    full_name='tensorflow.contrib.rpc.TestCaseService.IncrementTestShapes',
    index=0,
    containing_service=None,
    input_type=_TESTCASE,
    output_type=_TESTCASE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SleepForever',
    full_name='tensorflow.contrib.rpc.TestCaseService.SleepForever',
    index=1,
    containing_service=None,
    input_type=_TESTCASE,
    output_type=_TESTCASE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SometimesSleepForever',
    full_name='tensorflow.contrib.rpc.TestCaseService.SometimesSleepForever',
    index=2,
    containing_service=None,
    input_type=_TESTCASE,
    output_type=_TESTCASE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AlwaysFailWithInvalidArgument',
    full_name='tensorflow.contrib.rpc.TestCaseService.AlwaysFailWithInvalidArgument',
    index=3,
    containing_service=None,
    input_type=_TESTCASE,
    output_type=_TESTCASE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SometimesFailWithInvalidArgument',
    full_name='tensorflow.contrib.rpc.TestCaseService.SometimesFailWithInvalidArgument',
    index=4,
    containing_service=None,
    input_type=_TESTCASE,
    output_type=_TESTCASE,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TESTCASESERVICE)

DESCRIPTOR.services_by_name['TestCaseService'] = _TESTCASESERVICE

# @@protoc_insertion_point(module_scope)
