# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""Keras built-in datasets.
"""

from __future__ import print_function

from . import boston_housing
from . import cifar10
from . import cifar100
from . import fashion_mnist
from . import imdb
from . import mnist
from . import reuters

del print_function
