# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""VGG16 model for Keras.

# Reference

- [Very Deep Convolutional Networks for Large-Scale Image
Recognition](https://arxiv.org/abs/1409.1556)


"""

from __future__ import print_function

from tensorflow.python.keras.applications import VGG16
from tensorflow.python.keras.applications.densenet import decode_predictions
from tensorflow.python.keras.applications.imagenet_utils import preprocess_input

del print_function
