# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""Inception V3 model for Keras.

Note that the input image format for this model is different than for
the VGG16 and ResNet models (299x299 instead of 224x224),
and that the input preprocessing function is also different (same as Xception).

# Reference

- [Rethinking the Inception Architecture for Computer
Vision](http://arxiv.org/abs/1512.00567)


"""

from __future__ import print_function

from tensorflow.python.keras.applications import InceptionV3
from tensorflow.python.keras.applications.densenet import decode_predictions
from tensorflow.python.keras.applications.inception_v3 import preprocess_input

del print_function
